@@ -1,22 +1,40 @@
 # -*- test-case-name: klein.test.test_headers -*-
-# Copyright (c) 2011-2019. See LICENSE for details.
+# Copyright (c) 2011-2021. See LICENSE for details.
 
 """
 Tests for L{klein._headers}.
 """
 
+from abc import ABC, abstractmethod
 from collections import defaultdict
-from typing import AnyStr, Dict, Iterable, List, Optional, Text, Tuple, cast
+from string import ascii_letters
+from typing import (
+    AnyStr,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Tuple,
+    TypeVar,
+    cast,
+)
 
 from hypothesis import given
-from hypothesis.strategies import binary, iterables, text, tuples
+from hypothesis.strategies import (
+    binary,
+    characters,
+    composite,
+    iterables,
+    lists,
+    text,
+    tuples,
+)
 
-from ._strategies import ascii_text, latin1_text
-from ._trial import TestCase
 from .._headers import (
-    FrozenHTTPHeaders,
     HEADER_NAME_ENCODING,
     HEADER_VALUE_ENCODING,
+    FrozenHTTPHeaders,
     IHTTPHeaders,
     IMutableHTTPHeaders,
     MutableHTTPHeaders,
@@ -30,33 +48,82 @@ from .._headers import (
     normalizeRawHeaders,
     normalizeRawHeadersFrozen,
 )
+from ._trial import TestCase
 
 
 __all__ = ()
 
 
-def encodeName(name):
-    # type: (Text) -> Optional[bytes]
+T = TypeVar("T")
+DrawCallable = Callable[[Callable[..., T]], T]
+
+
+@composite
+def ascii_text(
+    draw: DrawCallable,
+    min_size: Optional[int] = 0,
+    max_size: Optional[int] = None,
+) -> str:  # pragma: no cover
+    """
+    A strategy which generates ASCII-encodable text.
+
+    @param min_size: The minimum number of characters in the text.
+        C{None} is treated as C{0}.
+
+    @param max_size: The maximum number of characters in the text.
+        Use C{None} for an unbounded size.
+    """
+    return cast(
+        str,
+        draw(
+            text(min_size=min_size, max_size=max_size, alphabet=ascii_letters)
+        ),
+    )
+
+
+@composite  # pragma: no cover
+def latin1_text(
+    draw: DrawCallable,
+    min_size: Optional[int] = 0,
+    max_size: Optional[int] = None,
+) -> str:
+    """
+    A strategy which generates ISO-8859-1-encodable text.
+
+    @param min_size: The minimum number of characters in the text.
+        C{None} is treated as C{0}.
+
+    @param max_size: The maximum number of characters in the text.
+        Use C{None} for an unbounded size.
+    """
+    return "".join(
+        draw(
+            lists(
+                characters(max_codepoint=255),
+                min_size=min_size,
+                max_size=max_size,
+            )
+        )
+    )
+
+
+def encodeName(name: str) -> Optional[bytes]:
     return name.encode(HEADER_NAME_ENCODING)
 
 
-def encodeValue(name):
-    # type: (Text) -> Optional[bytes]
+def encodeValue(name: str) -> Optional[bytes]:
     return name.encode(HEADER_VALUE_ENCODING)
 
 
-def decodeName(name):
-    # type: (bytes) -> Text
+def decodeName(name: bytes) -> str:
     return name.decode(HEADER_NAME_ENCODING)
 
 
-def decodeValue(name):
-    # type: (bytes) -> Text
+def decodeValue(name: bytes) -> str:
     return name.decode(HEADER_VALUE_ENCODING)
 
 
-def headerValueSanitize(value):
-    # type: (AnyStr) -> AnyStr
+def headerValueSanitize(value: AnyStr) -> AnyStr:
     """
     Sanitize a header value by replacing linear whitespace with spaces.
     """
@@ -77,68 +144,60 @@ class EncodingTests(TestCase):
     """
 
     @given(binary())
-    def test_headerNameAsBytesWithBytes(self, name):
-        # type: (bytes) -> None
+    def test_headerNameAsBytesWithBytes(self, name: bytes) -> None:
         """
         L{headerNameAsBytes} passes through L{bytes}.
         """
         self.assertIdentical(headerNameAsBytes(name), name)
 
     @given(latin1_text(min_size=1))
-    def test_headerNameAsBytesWithText(self, name):
-        # type: (Text) -> None
+    def test_headerNameAsBytesWithText(self, name: str) -> None:
         """
-        L{headerNameAsBytes} encodes L{Text} using L{HEADER_NAME_ENCODING}.
+        L{headerNameAsBytes} encodes L{str} using L{HEADER_NAME_ENCODING}.
         """
         rawName = encodeName(name)
         self.assertEqual(headerNameAsBytes(name), rawName)
 
     @given(binary())
-    def test_headerNameAsTextWithBytes(self, name):
-        # type: (bytes) -> None
+    def test_headerNameAsTextWithBytes(self, name: bytes) -> None:
         """
         L{headerNameAsText} decodes L{bytes} using L{HEADER_NAME_ENCODING}.
         """
         self.assertEqual(headerNameAsText(name), decodeName(name))
 
     @given(text(min_size=1))
-    def test_headerNameAsTextWithText(self, name):
-        # type: (Text) -> None
+    def test_headerNameAsTextWithText(self, name: str) -> None:
         """
-        L{headerNameAsText} passes through L{Text}.
+        L{headerNameAsText} passes through L{str}.
         """
         self.assertIdentical(headerNameAsText(name), name)
 
     @given(binary())
-    def test_headerValueAsBytesWithBytes(self, value):
-        # type: (bytes) -> None
+    def test_headerValueAsBytesWithBytes(self, value: bytes) -> None:
         """
         L{headerValueAsBytes} passes through L{bytes}.
         """
         self.assertIdentical(headerValueAsBytes(value), value)
 
     @given(latin1_text(min_size=1))
-    def test_headerValueAsBytesWithText(self, value):
-        # type: (Text) -> None
+    def test_headerValueAsBytesWithText(self, value: str) -> None:
         """
-        L{headerValueAsBytes} encodes L{Text} using L{HEADER_VALUE_ENCODING}.
+        L{headerValueAsBytes} encodes L{str} using L{HEADER_VALUE_ENCODING}.
         """
         rawValue = encodeValue(value)
         self.assertEqual(headerValueAsBytes(value), rawValue)
 
     @given(binary())
-    def test_headerValueAsTextWithBytes(self, value):
-        # type: (bytes) -> None
+    def test_headerValueAsTextWithBytes(self, value: bytes) -> None:
         """
         L{headerValueAsText} decodes L{bytes} using L{HEADER_VALUE_ENCODING}.
         """
         self.assertEqual(headerValueAsText(value), decodeValue(value))
 
     @given(text(min_size=1))
-    def test_headerValueAsTextWithText(self, value):
-        # type: (Text) -> None
+    def test_headerValueAsTextWithText(self, value: str) -> None:
         """
-        L{headerValueAsText} passes through L{Text}.
+        L{headerValueAsText} passes through L{str}.
         """
         self.assertIdentical(headerValueAsText(value), value)
 
@@ -148,8 +207,7 @@ class HeaderNameNormalizationTests(TestCase):
     Tests for header name normalization.
     """
 
-    def test_normalizeLowerCase(self):
-        # type: () -> None
+    def test_normalizeLowerCase(self) -> None:
         """
         L{normalizeHeaderName} normalizes header names to lower case.
         """
@@ -161,8 +219,7 @@ class RawHeadersConversionTests(TestCase):
     Tests for L{normalizeRawHeaders}.
     """
 
-    def test_pairWrongLength(self):
-        # type: () -> None
+    def test_pairWrongLength(self) -> None:
         """
         L{normalizeRawHeaders} raises L{ValueError} if the C{headerPairs}
         argument is not an iterable of 2-item iterables.
@@ -178,8 +235,7 @@ class RawHeadersConversionTests(TestCase):
             self.assertEqual(str(e), "header pair must be a 2-item iterable")
 
     @given(latin1_text())
-    def test_pairNameText(self, name):
-        # type: (Text) -> None
+    def test_pairNameText(self, name: str) -> None:
         """
         L{normalizeRawHeaders} converts ISO-8859-1-encodable text names into
         bytes.
@@ -193,8 +249,7 @@ class RawHeadersConversionTests(TestCase):
         )
 
     @given(latin1_text())
-    def test_pairValueText(self, value):
-        # type: (Text) -> None
+    def test_pairValueText(self, value: str) -> None:
         """
         L{normalizeRawHeaders} converts ISO-8859-1-encodable text values into
         bytes.
@@ -205,14 +260,16 @@ class RawHeadersConversionTests(TestCase):
         self.assertEqual(normalized, ((b"name", headerValueAsBytes(value)),))
 
 
-class GetValuesTestsMixIn(object):
+class GetValuesTestsMixIn(ABC):
     """
     Tests for utilities that access data from the C{RawHeaders} internal
     representation.
     """
 
-    def getValues(self, rawHeaders, name):
-        # type: (RawHeaders, AnyStr) -> Iterable[AnyStr]
+    @abstractmethod
+    def getValues(
+        self, rawHeaders: RawHeaders, name: AnyStr
+    ) -> Iterable[AnyStr]:
         """
         Look up the values for the given header name from the given raw
         headers.
@@ -221,19 +278,15 @@ class GetValuesTestsMixIn(object):
         cases that use it to specify how to perform this look-up in the
         implementation being tested.
         """
-        raise NotImplementedError(
-            "{} must implement getValues()".format(self.__class__)
-        )
 
-    def test_getBytesName(self):
-        # type: () -> None
+    def test_getBytesName(self) -> None:
         """
         C{getValues} returns an iterable of L{bytes} values for the
         given L{bytes} header name.
         """
         rawHeaders = ((b"a", b"1"), (b"b", b"2"), (b"c", b"3"), (b"B", b"TWO"))
 
-        normalized = defaultdict(list)  # type: Dict[bytes, List[bytes]]
+        normalized: Dict[bytes, List[bytes]] = defaultdict(list)
         for name, value in rawHeaders:
             normalized[normalizeHeaderName(name)].append(value)
 
@@ -241,11 +294,10 @@ class GetValuesTestsMixIn(object):
             cast(TestCase, self).assertEqual(
                 list(self.getValues(rawHeaders, name)),
                 values,
-                "header name: {!r}".format(name),
+                f"header name: {name!r}",
             )
 
-    def headerNormalize(self, value):
-        # type: (Text) -> Text
+    def headerNormalize(self, value: str) -> str:
         """
         Test hook for the normalization of header text values, which is a
         behavior Twisted has changed after version 18.9.0.
@@ -253,11 +305,10 @@ class GetValuesTestsMixIn(object):
         return value
 
     @given(iterables(tuples(ascii_text(min_size=1), latin1_text())))
-    def test_getTextName(self, textPairs):
-        # type: (Iterable[Tuple[Text, Text]]) -> None
+    def test_getTextName(self, textPairs: Iterable[Tuple[str, str]]) -> None:
         """
-        C{getValues} returns an iterable of L{Text} values for
-        the given L{Text} header name.
+        C{getValues} returns an iterable of L{str} values for
+        the given L{str} header name.
 
         This test only inserts Latin1 text into the header values, which is
         valid data.
@@ -266,7 +317,7 @@ class GetValuesTestsMixIn(object):
             (name, headerValueSanitize(value)) for name, value in textPairs
         )
 
-        textValues = defaultdict(list)  # type: Dict[Text, List[Text]]
+        textValues: Dict[str, List[str]] = defaultdict(list)
         for name, value in textHeaders:
             textValues[normalizeHeaderName(name)].append(value)
 
@@ -279,15 +330,16 @@ class GetValuesTestsMixIn(object):
             cast(TestCase, self).assertEqual(
                 list(self.getValues(rawHeaders, name)),
                 [self.headerNormalize(value) for value in _values],
-                "header name: {!r}".format(name),
+                f"header name: {name!r}",
             )
 
     @given(iterables(tuples(ascii_text(min_size=1), binary())))
-    def test_getTextNameBinaryValues(self, pairs):
-        # type: (Iterable[Tuple[Text, bytes]]) -> None
+    def test_getTextNameBinaryValues(
+        self, pairs: Iterable[Tuple[str, bytes]]
+    ) -> None:
         """
-        C{getValues} returns an iterable of L{Text} values for
-        the given L{Text} header name.
+        C{getValues} returns an iterable of L{str} values for
+        the given L{str} header name.
 
         This test only inserts binary data into the header values, which
         includes invalid data if you are a sane person, but arguably
@@ -300,7 +352,7 @@ class GetValuesTestsMixIn(object):
             for name, value in pairs
         )
 
-        binaryValues = defaultdict(list)  # type: Dict[Text, List[bytes]]
+        binaryValues: Dict[str, List[bytes]] = defaultdict(list)
         for name, value in rawHeaders:
             binaryValues[headerNameAsText(normalizeHeaderName(name))].append(
                 value
@@ -313,11 +365,10 @@ class GetValuesTestsMixIn(object):
                     self.headerNormalize(headerValueAsText(value))
                     for value in values
                 ),
-                "header name: {!r}".format(textName),
+                f"header name: {textName!r}",
             )
 
-    def test_getInvalidNameType(self):
-        # type: () -> None
+    def test_getInvalidNameType(self) -> None:
         """
         C{getValues} raises L{TypeError} when the given header name is of an
         unknown type.
@@ -326,7 +377,7 @@ class GetValuesTestsMixIn(object):
             TypeError, self.getValues, (), object()
         )
         cast(TestCase, self).assertRegex(
-            str(e), "name <object object at 0x[0-9a-f]+> must be text or bytes"
+            str(e), "name <object object at 0x[0-9a-f]+> must be str or bytes"
         )
 
 
@@ -336,8 +387,9 @@ class RawHeadersReadTests(GetValuesTestsMixIn, TestCase):
     representation.
     """
 
-    def getValues(self, rawHeaders, name):
-        # type: (RawHeaders, AnyStr) -> Iterable[AnyStr]
+    def getValues(
+        self, rawHeaders: RawHeaders, name: AnyStr
+    ) -> Iterable[AnyStr]:
         return getFromRawHeaders(normalizeRawHeadersFrozen(rawHeaders), name)
 
 
@@ -346,21 +398,20 @@ class FrozenHTTPHeadersTests(GetValuesTestsMixIn, TestCase):
     Tests for L{FrozenHTTPHeaders}.
     """
 
-    def getValues(self, rawHeaders, name):
-        # type: (RawHeaders, AnyStr) -> Iterable[AnyStr]
+    def getValues(
+        self, rawHeaders: RawHeaders, name: AnyStr
+    ) -> Iterable[AnyStr]:
         headers = FrozenHTTPHeaders(rawHeaders=rawHeaders)
         return headers.getValues(name=name)
 
-    def test_interface(self):
-        # type: () -> None
+    def test_interface(self) -> None:
         """
         L{FrozenHTTPHeaders} implements L{IHTTPHeaders}.
         """
         headers = FrozenHTTPHeaders(rawHeaders=())
         self.assertProvides(IHTTPHeaders, headers)
 
-    def test_defaultHeaders(self):
-        # type: () -> None
+    def test_defaultHeaders(self) -> None:
         """
         L{FrozenHTTPHeaders.rawHeaders} is empty by default.
         """
@@ -368,36 +419,36 @@ class FrozenHTTPHeadersTests(GetValuesTestsMixIn, TestCase):
         self.assertEqual(headers.rawHeaders, ())
 
 
-class MutableHTTPHeadersTestsMixIn(GetValuesTestsMixIn):
+class MutableHTTPHeadersTestsMixIn(GetValuesTestsMixIn, ABC):
     """
     Tests for L{IMutableHTTPHeaders} implementations.
     """
 
-    def assertRawHeadersEqual(self, rawHeaders1, rawHeaders2):
-        # type: (RawHeaders, RawHeaders) -> None
+    def assertRawHeadersEqual(
+        self, rawHeaders1: RawHeaders, rawHeaders2: RawHeaders
+    ) -> None:
         cast(TestCase, self).assertEqual(rawHeaders1, rawHeaders2)
 
-    def headers(self, rawHeaders):
-        # type: (RawHeaders) -> IMutableHTTPHeaders
-        raise NotImplementedError(
-            "{} must implement headers()".format(self.__class__)
-        )
+    @abstractmethod
+    def headers(self, rawHeaders: RawHeaders) -> IMutableHTTPHeaders:
+        """
+        Given a L{RawHeaders}, return an L{IMutableHTTPHeaders}.
+        """
 
-    def getValues(self, rawHeaders, name):
-        # type: (RawHeaders, AnyStr) -> Iterable[AnyStr]
+    def getValues(
+        self, rawHeaders: RawHeaders, name: AnyStr
+    ) -> Iterable[AnyStr]:
         headers = self.headers(rawHeaders=rawHeaders)
         return headers.getValues(name=name)
 
-    def test_interface(self):
-        # type: () -> None
+    def test_interface(self) -> None:
         """
         Class implements L{IMutableHTTPHeaders}.
         """
         headers = self.headers(rawHeaders=())
         cast(TestCase, self).assertProvides(IMutableHTTPHeaders, headers)
 
-    def test_rawHeaders(self):
-        # type: () -> None
+    def test_rawHeaders(self) -> None:
         """
         L{IMutableHTTPHeaders.rawHeaders} equals the raw headers passed at init
         time as a tuple.
@@ -406,8 +457,7 @@ class MutableHTTPHeadersTestsMixIn(GetValuesTestsMixIn):
         headers = self.headers(rawHeaders=rawHeaders)
         self.assertRawHeadersEqual(headers.rawHeaders, tuple(rawHeaders))
 
-    def test_removeBytesName(self):
-        # type: () -> None
+    def test_removeBytesName(self) -> None:
         """
         L{IMutableHTTPHeaders.remove} removes all values for the given L{bytes}
         header name.
@@ -420,22 +470,20 @@ class MutableHTTPHeadersTestsMixIn(GetValuesTestsMixIn):
             headers.rawHeaders, ((b"a", b"1"), (b"c", b"3"))
         )
 
-    def test_removeTextName(self):
-        # type: () -> None
+    def test_removeTextName(self) -> None:
         """
-        L{IMutableHTTPHeaders.remove} removes all values for the given L{Text}
+        L{IMutableHTTPHeaders.remove} removes all values for the given L{str}
         header name.
         """
         rawHeaders = ((b"a", b"1"), (b"b", b"2a"), (b"c", b"3"), (b"b", b"2b"))
         headers = self.headers(rawHeaders=rawHeaders)
-        headers.remove(name=u"b")
+        headers.remove(name="b")
 
         self.assertRawHeadersEqual(
             headers.rawHeaders, ((b"a", b"1"), (b"c", b"3"))
         )
 
-    def test_removeInvalidNameType(self):
-        # type: () -> None
+    def test_removeInvalidNameType(self) -> None:
         """
         L{IMutableHTTPHeaders.remove} raises L{TypeError} when the given header
         name is of an unknown type.
@@ -446,11 +494,10 @@ class MutableHTTPHeadersTestsMixIn(GetValuesTestsMixIn):
             TypeError, headers.remove, object()
         )
         cast(TestCase, self).assertRegex(
-            str(e), "name <object object at 0x[0-9a-f]+> must be text or bytes"
+            str(e), "name <object object at 0x[0-9a-f]+> must be str or bytes"
         )
 
-    def test_addValueBytesName(self):
-        # type: () -> None
+    def test_addValueBytesName(self) -> None:
         """
         L{IMutableHTTPHeaders.addValue} adds the given L{bytes} value for the
         given L{bytes} header name.
@@ -463,52 +510,48 @@ class MutableHTTPHeadersTestsMixIn(GetValuesTestsMixIn):
             headers.rawHeaders, ((b"a", b"1"), (b"b", b"2a"), (b"b", b"2b"))
         )
 
-    def test_addValueTextName(self):
-        # type: () -> None
+    def test_addValueTextName(self) -> None:
         """
-        L{IMutableHTTPHeaders.addValue} adds the given L{Text} value for the
-        given L{Text} header name.
+        L{IMutableHTTPHeaders.addValue} adds the given L{str} value for the
+        given L{str} header name.
         """
         rawHeaders = ((b"a", b"1"), (b"b", b"2a"))
         headers = self.headers(rawHeaders=rawHeaders)
-        headers.addValue(name=u"b", value=u"2b")
+        headers.addValue(name="b", value="2b")
 
         self.assertRawHeadersEqual(
             headers.rawHeaders, ((b"a", b"1"), (b"b", b"2a"), (b"b", b"2b"))
         )
 
-    def test_addValueBytesNameTextValue(self):
-        # type: () -> None
+    def test_addValueBytesNameTextValue(self) -> None:
         """
         L{IMutableHTTPHeaders.addValue} raises L{TypeError} when the given
-        header name is L{bytes} and the given value is L{Text}.
+        header name is L{bytes} and the given value is L{str}.
         """
         headers = self.headers(rawHeaders=())
 
         e = cast(TestCase, self).assertRaises(
-            TypeError, headers.addValue, b"a", u"1"
+            TypeError, headers.addValue, b"a", "1"
         )
         cast(TestCase, self).assertRegex(
             str(e), "value u?'1' must be bytes to match name b?'a'"
         )
 
-    def test_addValueTextNameBytesValue(self):
-        # type: () -> None
+    def test_addValueTextNameBytesValue(self) -> None:
         """
         L{IMutableHTTPHeaders.addValue} raises L{TypeError} when the given
-        header name is L{Text} and the given value is L{bytes}.
+        header name is L{str} and the given value is L{bytes}.
         """
         headers = self.headers(rawHeaders=())
 
         e = cast(TestCase, self).assertRaises(
-            TypeError, headers.addValue, u"a", b"1"
+            TypeError, headers.addValue, "a", b"1"
         )
         cast(TestCase, self).assertRegex(
-            str(e), "value b?'1' must be text to match name u?'a'"
+            str(e), "value b?'1' must be str to match name u?'a'"
         )
 
-    def test_addValueInvalidNameType(self):
-        # type: () -> None
+    def test_addValueInvalidNameType(self) -> None:
         """
         L{IMutableHTTPHeaders.addValue} raises L{TypeError} when the given
         header name is of an unknown type.
@@ -519,7 +562,7 @@ class MutableHTTPHeadersTestsMixIn(GetValuesTestsMixIn):
             TypeError, headers.addValue, object(), b"1"
         )
         cast(TestCase, self).assertRegex(
-            str(e), "name <object object at 0x[0-9a-f]+> must be text or bytes"
+            str(e), "name <object object at 0x[0-9a-f]+> must be str or bytes"
         )
 
 
@@ -528,12 +571,10 @@ class MutableHTTPHeadersTests(MutableHTTPHeadersTestsMixIn, TestCase):
     Tests for L{MutableHTTPHeaders}.
     """
 
-    def headers(self, rawHeaders):
-        # type: (RawHeaders) -> IMutableHTTPHeaders
+    def headers(self, rawHeaders: RawHeaders) -> IMutableHTTPHeaders:
         return MutableHTTPHeaders(rawHeaders=rawHeaders)
 
-    def test_defaultHeaders(self):
-        # type: () -> None
+    def test_defaultHeaders(self) -> None:
         """
         L{MutableHTTPHeaders.rawHeaders} is empty by default.
         """
