@@ -1,33 +1,58 @@
 # -*- test-case-name: klein.test.test_request_compat -*-
-# Copyright (c) 2011-2019. See LICENSE for details.
+# Copyright (c) 2011-2021. See LICENSE for details.
 
 """
 Tests for L{klein._irequest}.
 """
 
+import functools
 from string import ascii_uppercase
-from typing import Optional, Text
+from types import MappingProxyType
+from typing import (
+    Any,
+    Awaitable,
+    Callable,
+    Coroutine,
+    Mapping,
+    Sequence,
+    TypeVar,
+)
 
 from hyperlink import DecodedURL, EncodedURL
-
+from hyperlink.hypothesis import decoded_urls
 from hypothesis import given
 from hypothesis.strategies import binary, text
 
-from twisted.web.http_headers import Headers
+from twisted.internet.defer import ensureDeferred
 from twisted.web.iweb import IRequest
 
-from ._strategies import decoded_urls
-from ._trial import TestCase
-from .test_resource import requestMock
 from .._headers import IHTTPHeaders
 from .._message import FountAlreadyAccessedError
 from .._request import IHTTPRequest
 from .._request_compat import HTTPRequestWrappingIRequest
+from ._trial import TestCase
+from .test_resource import MockRequest
 
 
 __all__ = ()
 
 
+emptyMapping: Mapping[Any, Any] = MappingProxyType({})
+
+_T = TypeVar("_T")
+_R = TypeVar("_R")
+
+
+def ensuringDeferred(
+    fn: Callable[[_T], Coroutine[Any, Any, _R]]
+) -> Callable[[_T], Awaitable[_R]]:
+    @functools.wraps(fn)
+    def wrapper(self: _T) -> Awaitable[_R]:
+        return ensureDeferred(fn(self))
+
+    return wrapper
+
+
 class HTTPRequestWrappingIRequestTests(TestCase):
     """
     Tests for L{HTTPRequestWrappingIRequest}.
@@ -35,16 +60,15 @@ class HTTPRequestWrappingIRequestTests(TestCase):
 
     def legacyRequest(
         self,
-        path=b"/",  # type: bytes
-        method=b"GET",  # type: bytes
-        host=b"localhost",  # type: bytes
-        port=8080,  # type: int
-        isSecure=False,  # type: bool
-        body=None,  # type: Optional[bytes]
-        headers=None,  # type: Optional[Headers]
-    ):
-        # type: (...) -> IRequest
-        return requestMock(
+        path: bytes = b"/",
+        method: bytes = b"GET",
+        host: bytes = b"localhost",
+        port: int = 8080,
+        isSecure: bool = False,
+        body: bytes = b"",
+        headers: Mapping[bytes, Sequence[bytes]] = emptyMapping,
+    ) -> IRequest:
+        return MockRequest(
             path=path,
             method=method,
             host=host,
@@ -54,8 +78,7 @@ class HTTPRequestWrappingIRequestTests(TestCase):
             headers=headers,
         )
 
-    def test_interface(self):
-        # type: () -> None
+    def test_interface(self) -> None:
         """
         L{HTTPRequestWrappingIRequest} implements L{IHTTPRequest}.
         """
@@ -63,8 +86,7 @@ class HTTPRequestWrappingIRequestTests(TestCase):
         self.assertProvides(IHTTPRequest, request)
 
     @given(text(alphabet=ascii_uppercase, min_size=1))
-    def test_method(self, methodText):
-        # type: (Text) -> None
+    def test_method(self, methodText: str) -> None:
         """
         L{HTTPRequestWrappingIRequest.method} matches the underlying legacy
         request method.
@@ -74,8 +96,7 @@ class HTTPRequestWrappingIRequestTests(TestCase):
         self.assertEqual(request.method, methodText)
 
     @given(decoded_urls())
-    def test_uri(self, url):
-        # type: (DecodedURL) -> None
+    def test_uri(self, url: DecodedURL) -> None:
         """
         L{HTTPRequestWrappingIRequest.uri} matches the underlying legacy
         request URI.
@@ -85,12 +106,10 @@ class HTTPRequestWrappingIRequestTests(TestCase):
         assert uri.port is not None  # Tells mypy it's not an Optional
 
         path = (
-            uri.replace(scheme=u"", host=u"", port=None)
-            .asText()
-            .encode("ascii")
+            uri.replace(scheme="", host="", port=None).asText().encode("ascii")
         )
         legacyRequest = self.legacyRequest(
-            isSecure=(uri.scheme == u"https"),
+            isSecure=(uri.scheme == "https"),
             host=uri.host.encode("ascii"),
             port=uri.port,
             path=path,
@@ -101,17 +120,16 @@ class HTTPRequestWrappingIRequestTests(TestCase):
         requestURINormalized = request.uri.asURI()
 
         # Needed because non-equal URLs can render as the same strings
-        def strURL(url):
-            # type: (EncodedURL) -> Text
+        def strURL(url: EncodedURL) -> str:
             return (
-                u"URL(scheme={url.scheme!r}, "
-                u"userinfo={url.userinfo!r}, "
-                u"host={url.host!r}, "
-                u"port={url.port!r}, "
-                u"path={url.path!r}, "
-                u"query={url.query!r}, "
-                u"fragment={url.fragment!r}, "
-                u"rooted={url.rooted})"
+                "URL(scheme={url.scheme!r}, "
+                "userinfo={url.userinfo!r}, "
+                "host={url.host!r}, "
+                "port={url.port!r}, "
+                "path={url.path!r}, "
+                "query={url.query!r}, "
+                "fragment={url.fragment!r}, "
+                "rooted={url.rooted})"
             ).format(url=url)
 
         self.assertEqual(
@@ -122,8 +140,7 @@ class HTTPRequestWrappingIRequestTests(TestCase):
             ),
         )
 
-    def test_headers(self):
-        # type: () -> None
+    def test_headers(self) -> None:
         """
         L{HTTPRequestWrappingIRequest.headers} returns an
         L{HTTPRequestWrappingIRequest} containing the underlying legacy request
@@ -133,8 +150,7 @@ class HTTPRequestWrappingIRequestTests(TestCase):
         request = HTTPRequestWrappingIRequest(request=legacyRequest)
         self.assertProvides(IHTTPHeaders, request.headers)
 
-    def test_bodyAsFountTwice(self):
-        # type: () -> None
+    def test_bodyAsFountTwice(self) -> None:
         """
         L{HTTPRequestWrappingIRequest.bodyAsFount} raises
         L{FountAlreadyAccessedError} if called more than once.
@@ -145,20 +161,19 @@ class HTTPRequestWrappingIRequestTests(TestCase):
         self.assertRaises(FountAlreadyAccessedError, request.bodyAsFount)
 
     @given(binary())
-    def test_bodyAsBytes(self, data):
-        # type: (bytes) -> None
+    def test_bodyAsBytes(self, data: bytes) -> None:
         """
         L{HTTPRequestWrappingIRequest.bodyAsBytes} matches the underlying
         legacy request body.
         """
         legacyRequest = self.legacyRequest(body=data)
         request = HTTPRequestWrappingIRequest(request=legacyRequest)
-        body = self.successResultOf(request.bodyAsBytes())
+        body = self.successResultOf(ensureDeferred(request.bodyAsBytes()))
 
         self.assertEqual(body, data)
 
-    def test_bodyAsBytesCached(self):
-        # type: () -> None
+    @ensuringDeferred
+    async def test_bodyAsBytesCached(self) -> None:
         """
         L{HTTPRequestWrappingIRequest.bodyAsBytes} called twice returns the
         same object both times.
@@ -166,7 +181,7 @@ class HTTPRequestWrappingIRequestTests(TestCase):
         data = b"some data"
         legacyRequest = self.legacyRequest(body=data)
         request = HTTPRequestWrappingIRequest(request=legacyRequest)
-        body1 = self.successResultOf(request.bodyAsBytes())
-        body2 = self.successResultOf(request.bodyAsBytes())
+        body1 = await request.bodyAsBytes()
+        body2 = await request.bodyAsBytes()
 
         self.assertIdentical(body1, body2)
