@@ -1,8 +1,8 @@
 import os
 from io import BytesIO
 from types import MappingProxyType
-from typing import Any, Callable, List, Mapping, Optional, Sequence, cast
-from unittest.mock import Mock, call
+from typing import Any, List, Mapping, Optional, Sequence, cast
+from unittest.mock import ANY, Mock, call
 from urllib.parse import parse_qs
 
 from werkzeug.exceptions import NotFound
@@ -250,11 +250,11 @@ class KleinResourceEqualityTests(SynchronousTestCase, EqualityTestsMixin):
 
     _another = _Another()
 
-    def anInstance(self) -> Callable[[], KleinResource]:
-        return self._one.oneKlein.resource
+    def anInstance(self) -> KleinResource:
+        return self._one.oneKlein.resource()
 
-    def anotherInstance(self) -> Callable[[], KleinResource]:
-        return self._another.anotherKlein.resource
+    def anotherInstance(self) -> KleinResource:
+        return self._another.anotherKlein.resource()
 
 
 class KleinResourceTests(SynchronousTestCase):
@@ -575,9 +575,10 @@ class KleinResourceTests(SynchronousTestCase):
         app = self.app
 
         request = MockRequest(b"/__init__.py")
-        expected = open(
+        with open(
             os.path.join(os.path.dirname(__file__), "__init__.py"), "rb"
-        ).read()
+        ) as f:
+            expected = f.read()
 
         @app.route("/", branch=True)
         def root(request: IRequest) -> KleinRenderable:
@@ -593,9 +594,10 @@ class KleinResourceTests(SynchronousTestCase):
         app = self.app
 
         request = MockRequest(b"/static/__init__.py")
-        expected = open(
+        with open(
             os.path.join(os.path.dirname(__file__), "__init__.py"), "rb"
-        ).read()
+        ) as f:
+            expected = f.read()
 
         @app.route("/static/", branch=True)
         def root(request: IRequest) -> KleinRenderable:
@@ -642,7 +644,7 @@ class KleinResourceTests(SynchronousTestCase):
         request.setHeader.assert_has_calls(
             [
                 call(b"Content-Type", b"text/html; charset=utf-8"),
-                call(b"Content-Length", b"258"),
+                call(b"Content-Length", ANY),
                 call(b"Location", b"http://localhost:8080/foo/"),
             ]
         )
